local AddonName, a = ...
local L = a.Localize
local s = SpellFlashAddon
local c = BittensSpellFlashLibrary

a.Rotations = {}
c.RegisterForEvents(a)
s.Spam[AddonName] = function()
    a.UpdateBCMVisibility()
    c.Flash(a)
end

------------------------------------------------------------------------- 4pT13
a.StolenTimeRemaining = 10

local function auraApplied4pT13(spellID, target, spellSchool)
    if not c.WearingSet(4, "T13") then
    
        a.StolenTimeRemaining = 0
    elseif spellID == c.GetID("Arcane Power")
        or spellID == c.GetID("Combustion")
        or spellID == c.GetID("Icy Veins") then
        
        a.StolenTimeRemaining = 10
        c.Debug("Event", "Stolen Time will reset", a.StolenTimeRemaining)
    elseif spellID == c.GetID("Stolen Time")
        and a.StolenTimeRemaining > 0
        and not s.Buff(c.GetID("Arcane Power"), "player")
        and not s.MyDebuff(c.GetID("Combustion"))
        and not s.Buff(c.GetID("Icy Veins"), "player") then
            
        a.StolenTimeRemaining = a.StolenTimeRemaining - 1
        c.Debug("Event", "Stolen Time bump", a.StolenTimeRemaining)
    end
end

local function auraRemoved4pT13(spellID, target, spellSchool)
    if spellID == c.GetID("Stolen Time") then
        a.StolenTimeRemaining = 10
        c.Debug("Event", "Stolen Time reset", 10)
    end
end

------------------------------------------------------------------------ Arcane
a.Phase = "neutral"
a.UnstackedMaxMana = 1000000000

local function setBlastColor(color)
    a.spells["Arcane Blast"].FlashColor = color
end

local function flashBlast(color)
    setBlastColor(color)
    a.Flash("Arcane Blast")
end

local function flashNeutralAround(percent, manaRatio, warningRatio)
    if warningRatio and manaRatio >= warningRatio then
        setBlastColor("red")
    else
        setBlastColor(nil)
    end
    if manaRatio > percent / 100 then
        a.FlashAll("Arcane Blast", "Conjure Mana Gem")
    else
        a.Flash("Arcane Missiles", "Arcane Blast")
    end
end

a.Rotations.Arcane = {
    Spec = 1,
    OffSwitch = "arcane_off",
    
    FlashInCombat = function()
        
        ------------------------------------------------- mana calculations
        local mana = s.Power("player")
        local casting = s.CastingOrChannelingName(nil, "player")
        if casting ~= nil then
            mana = mana - s.SpellCost(casting)
        end
        
        local stackedMana = 
            78 * 15 * s.BuffStack(c.GetID("Combat Mind LFR"), "player")
                + 88 * 15 * s.BuffStack(c.GetID("Combat Mind"), "player")
                + 99 * 15 * s.BuffStack(c.GetID("Combat Mind Heroic"), "player")
        if s.Buff(c.FIVE_PERCENT_BUFFS, "player") then
            stackedMana = stackedMana * 1.05
        end
        local maxMana = s.MaxPower("player") - stackedMana
        if maxMana < a.UnstackedMaxMana then
            a.UnstackedMaxMana = maxMana
        end
        maxMana = a.UnstackedMaxMana + stackedMana
        local manaRatio = mana / maxMana
        
        ------------------------------------------------- phase transitions
        local abStack = s.DebuffStack(c.GetID("Arcane Blast"), "player")
        if abStack < 4 and casting == s.SpellName(c.GetID("Arcane Blast")) then
            abStack = abStack + 1
        end
        
        if a.Phase == "neutral"
            and c.GetCooldown("Evocation") < a.GetConfig("burn_length") then
            
            a.Phase = "burn transition"
        end
        
        if a.Phase == "burn transition" and c.HasBuff("Arcane Power") then
            a.Phase = "burn"
        end
        
        if a.Phase == "burn"
            and manaRatio < a.GetConfig("evocate_percent") / 100 then
            
            a.Phase = "neutral transition"
        end
        
        if casting == s.SpellName(c.GetID("Evocation")) then
            a.Phase = "neutral"
        end
        
        ---------------------------------------------------------- flashing
        a.FlashAll(
            "Presence of Mind", "Mirror Image", "Counterspell", "Spellsteal")
        if a.Phase ~= "burn" or not c.HasBuff("Arcane Power") then
            a.Flash("Flame Orb")
        end
        
        if a.Phase == "neutral" then
            flashNeutralAround(a.GetConfig("missiles_percent"), manaRatio, 1)
            local apCool = 120
            if c.WearingSet(4, "T13") then
                apCool = apCool - (10 - a.StolenTimeRemaining) * 7
            end
            apCool = apCool * (1 - .125 * s.TalentRank(44379)) -- Arcane Flows
            if c.GetCooldown("Evocation") - a.GetConfig("burn_length") 
                > apCool then
            
                a.Flash("Arcane Power")
            end
            
        elseif a.Phase == "burn transition" then
            flashBlast("yellow")
            flashNeutralAround(a.GetConfig("missiles_percent"), manaRatio, 1)
            if abStack == 4 and maxMana - mana >= 13000 then -- mana gem
                if a.Flash("Arcane Power") then
                    a.Flash("Mana Gem")
                end
            end
            
        elseif a.Phase == "burn" then
            flashBlast()
            if c.GetBuffDuration("Arcane Power", true) > 10 then
                a.Flash("Mana Gem")
            end
            
        elseif a.Phase == "neutral transition" then
            if not a.Flash("Evocation") then
                flashNeutralAround(
                    a.GetConfig("evocate_percent")
                        - (100 - a.GetConfig("missiles_percent")),
                    manaRatio)
            end
        end
    end,
    
    FlashOutOfCombat = function()
        a.Flash("Conjure Mana Gem")
    end,
    
    FlashAlways = function()
        a.FlashAll(
            "Focus Magic",
            "Mage Armor",
            "Arcane Brilliance",
            "Dalaran Brilliance")
    end,
    
    AuraApplied = auraApplied4pT13,
    
    AuraRemoved = auraRemoved4pT13, 
    
    CastSucceeded = function(info)
        if c.InfoMatches(info, "Focus Magic") then
            a.FocusMagicTarget = target
            c.Debug("Event", "Focus Magic target:", target)
        end
    end,
    
    LeftCombat = function()
        a.Phase = "neutral"
        a.UnstackedMaxMana = 1000000000
        c.Debug("Event", "Left combat")
    end
}

-------------------------------------------------------------------------- Fire
a.ScorchDelay = 0

local function dangerOfHotStreakOverwrite()
    local startDelay = c.GetBusyTime(
        false, 
        "Fireball", 
        "Scorch", 
        "Flame Orb", 
        "Living Bomb", 
        "Pyroblast", 
        "Pyroblast!")
    local endDelay = startDelay + c.GetCastTime("Fireball")
    return c.HasBuff("Hot Streak")
        and (c.CountLandings("Fireball", startDelay, endDelay) > 0
            or c.CountLandings("Pyroblast!", startDelay, endDelay) > 0
            or c.CountLandings("Scorch", startDelay, endDelay) > 0
            or c.CountLandings("Pyroblast", startDelay, endDelay) > 0)
end

a.Rotations.Fire = {
    Spec = 2,
    OffSwitch = "fire_off",
    
    FlashInCombat = function()
        a.FlashAll(
            "Combustion",
            "Fire Blast Impact",
            "Counterspell",
            --"Mana Gem", -- uncomment for Iverie's mana management
            "Spellsteal")
        
        if dangerOfHotStreakOverwrite() then
            a.FlashAll("Pyroblast", "Pyroblast!")
        else
            a.Flash(
                "Scorch to Apply Critical Mass",
                "Living Bomb",
                "Flame Orb",
                "Mirror Image",
                "Pyroblast",
                "Pyroblast!",
                "Scorch to Refresh Critical Mass",
                "Evocation at 10%", -- comment for Iverie's mana management
                --"Evocation at 40%", -- uncomment for Iverie's mana management
                --"Mage Armor at 5%", -- uncomment for Iverie's mana management
                --"Molten Armor at 45%", -- uncomment for Iverie's mana management
                "Fireball")
                --"Scorch")
        end
    end,
    
    FlashOutOfCombat = function()
        a.Flash(
            --"Molten Armor", -- uncomment for Iverie's mana management
            "Conjure Mana Gem")
    end,
    
    FlashAlways = function()
        a.FlashAll(
            "Molten Armor", -- comment for Iverie's mana management
            "Arcane Brilliance",
            "Dalaran Brilliance")
    end,
    
    SpellDamage = function(
        spellID, target, amount, critical, spellSchool, damageSchool, periodic)
        
        if not periodic
            and (spellID == c.GetID("Scorch")
                or spellID == c.GetID("Pyroblast!")
                or spellID == c.GetID("Pyroblast")) then
            
            a.ScorchDelay = GetTime() + .6
            c.Debug("Event", "Suppress Scorch for .6 seconds")
        end
    end,
}

------------------------------------------------------------------------- Frost
local blizzFingerCount = -1
a.FingerCount = blizzFingerCount

a.Rotations.Frost = {
    Spec = 3,
    OffSwitch = "frost_off",
    
    FlashInCombat = function()
        a.FlashAll(
            "Mirror Image", 
            "Icy Veins", 
            "Cold Snap", 
            "Counterspell",
            "Spellsteal")
        a.Flash(
            "Frostfire Orb",
            "Deep Freeze",
            "Freeze",
            "Freeze on Pet Bar",
            "Instant Frozen Frostfire Bolt",
            "Frozen Ice Lance",
            "Evocation at 10%",
            "Frostbolt")
    end,
    
    FlashOutOfCombat = function()
        a.Flash("Conjure Mana Gem")
    end,
    
    FlashAlways = function()
        a.FlashAll(
            "Molten Armor", 
            "Summon Water Elemental", 
            "Arcane Brilliance", 
            "Dalaran Brilliance")
    end,
    
    CastSucceeded = function(info)
        if c.InfoMatches(
            info, "Ice Lance", "Frostfire Bolt", "Deep Freeze") then
            
            a.FingerCount = math.max(0, a.FingerCount - 1)
            c.Debug("Event", "Pretend FoF is now at", a.FingerCount)
        end
    end,
    
    AuraApplied = function(spellID, target, spellSchool)
        if spellID == c.GetID("Fingers of Frost") then
            local stack = c.GetBuffStack("Fingers of Frost")
            if a.FingerCount == blizzFingerCount - 1 then
                a.FingerCount = stack - 1
            else
                a.FingerCount = stack
            end
            c.Debug("Event",
                "Gained FoF. Stack =", stack, "Pretend =", a.FingerCount)
            blizzFingerCount = stack
        else
            auraApplied4pT13(spellID, target, spellSchool)
        end
    end,
    
    AuraRemoved = function(spellID, target, spellSchool)
        if spellID == c.GetID("Fingers of Frost") then
            local stack = c.GetBuffStack("Fingers of Frost")
            a.FingerCount = stack
            c.Debug("Event", "Lost FoF. Stack =", stack)
            blizzFingerCount = stack
        else
            auraRemoved4pT13(spellID, target, spellSchool)
        end
    end, 
}
